using IdentityModel.OidcClient;
using IdentityModel.OidcClient.Browser;
using System;
using System.Drawing;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace TESTING.OIDCWINFORM
{
    public partial class Form1 : Form
    {
        //private OidcClient _oidcClient;
        private OidcClient _JCoidcClient;
        private OidcClient _OLoidcClient;
        private LoginResult _loginResult;
        public Form1()
        {
            InitializeComponent();
            // Set up the OIDC client configuration
            var JCoptions = new OidcClientOptions
            {
                Authority = JCAuth.Text.ToString(), // Replace with your IDP's authority URL
                ClientId = JCID.Text.ToString(), // Replace with your client ID
                ClientSecret = JCSecret.Text.ToString(), // Replace with your client secret
                RedirectUri = commonRedirect.Text.ToString(), // Replace with your redirect URI
                Scope = commonScope.Text.ToString(), // Replace with desired scopes
                FilterClaims = false,
                Browser = new SystemBrowser()
            };
            var OLoptions = new OidcClientOptions
            {
                Authority = OLAuth.Text.ToString(), // Replace with your IDP's authority URL
                ClientId = OLID.Text.ToString(), // Replace with your client ID
                ClientSecret = OLSecret.Text.ToString(), // Replace with your client secret
                RedirectUri = commonRedirect.Text.ToString(), // Replace with your redirect URI
                Scope = commonScope.Text.ToString(), // Replace with desired scopes
                FilterClaims = false,
                Browser = new SystemBrowser()
            };
            _JCoidcClient = new OidcClient(JCoptions);
            _OLoidcClient = new OidcClient(OLoptions);  
        }

        private async void JCLoginButton_Click(object sender, EventArgs e)
        {
            _loginResult = await _JCoidcClient.LoginAsync(new LoginRequest(), CancellationToken.None);

            if (_loginResult.IsError)
            {
                MessageBox.Show($"Error: {_loginResult.Error}");
            }
            else
            {
                MessageBox.Show($"Success: {_loginResult.User.Identity.AuthenticationType}");
                // Hide the LoginForm form
                Hide();

                // Retrieve the email claim
                var userEmailClaim = _loginResult.User.FindFirst(c => c.Type == "email")?.Value;

                // Pass the user email to the SecureForm constructor
                var secureForm = new secureform(userEmailClaim, _loginResult.User.Identity.AuthenticationType);
                secureForm.ShowDialog();

                // Close the application after the SecureForm is closed
                Close();
            }
        }
        private async void OLLoginButton_Click(object sender, EventArgs e)
        {
             _loginResult = await _OLoidcClient.LoginAsync(new LoginRequest(), CancellationToken.None);

            if (_loginResult.IsError)
            {
                MessageBox.Show($"Error: {_loginResult.Error}");
            }
            else
            {
                MessageBox.Show($"Success: {_loginResult.User.Identity.AuthenticationType}");
                // Hide the LoginForm form
                Hide();

                // Retrieve the email claim
                var userEmailClaim = _loginResult.User.FindFirst(c => c.Type == "email")?.Value;

                // Pass the user email to the SecureForm constructor
                var secureForm = new secureform(userEmailClaim, _loginResult.User.Identity.AuthenticationType);
                secureForm.ShowDialog();

                // Close the application after the SecureForm is closed
                Close();
            }
        }
        private class SystemBrowser : IBrowser
        {
            public async Task<BrowserResult> InvokeAsync(BrowserOptions options, CancellationToken cancellationToken = default)
            {
                using (var form = new Form())
                {
                    form.Text = "Redirect to the IdP you clicked";
                    form.Size = new Size(800, 600);
                    form.StartPosition = FormStartPosition.CenterScreen;

                    var browser = new WebBrowser();
                    browser.Dock = DockStyle.Fill;
                    browser.ScriptErrorsSuppressed = true   ;   

                    var tcs = new TaskCompletionSource<string>();

                    browser.Navigated += (s, e) =>
                    {
                        MessageBox.Show("Browser nav has changed!\n\n"+e.Url.AbsoluteUri);
                        if (e.Url.AbsoluteUri.StartsWith("http://localhost"))
                        {
                            tcs.SetResult(e.Url.AbsoluteUri);
                            form.Close();
                        }
                        else if (e.Url.AbsoluteUri.StartsWith("https://console.jumpcloud.com/userconsole#/?error=noAccess"))
                        {
                            MessageBox.Show("Getting a 'NO ACCESS' type return from JC");
                        }
                    };

                    form.Controls.Add(browser);

                    form.Shown += (s, e) =>
                    {
                        browser.Navigate(options.StartUrl);
                    };

                    form.ShowDialog();

                    var result = new BrowserResult
                    {
                        
                        Response = await tcs.Task,
                        ResultType = BrowserResultType.Success
                    };

                    return result;
                }
            }
        }
    }
}