using IdentityModel.OidcClient.Browser;
using IdentityModel.OidcClient;
using System;
using System.Drawing;
using System.Security.Cryptography;
using System.Threading.Tasks;
using System.Threading;
using Wisej.Web;
using MessageBox = Wisej.Web.MessageBox;

namespace TESTING.WJWEBCORE
{
    public partial class Window1 : Form
    {
        private OidcClient _JCoidcClient;
        private OidcClient _OLoidcClient;
        private LoginResult _loginResult;

        public Window1()
        {
            InitializeComponent();

            // Set up the OIDC client configuration for JC
            var JCoptions = new OidcClientOptions
            {
                Authority = JCAuth.Text.ToString(), // Replace with your IDP's authority URL
                ClientId = JCID.Text.ToString(), // Replace with your client ID
                ClientSecret = JCSecret.Text.ToString(), // Replace with your client secret
                RedirectUri = commonRedirect.Text.ToString(), // Replace with your redirect URI
                Scope = commonScope.Text.ToString(), // Replace with desired scopes
                FilterClaims = false,
                Browser = new SystemBrowser(webBrowser1)
            };
            _JCoidcClient = new OidcClient(JCoptions);

            // Set up the OIDC client configuration for OL
            var OLoptions = new OidcClientOptions
            {
                Authority = OLAuth.Text.ToString(), // Replace with your IDP's authority URL
                ClientId = OLID.Text.ToString(), // Replace with your client ID
                ClientSecret = OLSecret.Text.ToString(), // Replace with your client secret
                RedirectUri = commonRedirect.Text.ToString(), // Replace with your redirect URI
                Scope = commonScope.Text.ToString(), // Replace with desired scopes
                FilterClaims = false,
                Browser = new SystemBrowser(webBrowser1)
            };
            _OLoidcClient = new OidcClient(OLoptions);
        }

        private async void JCLoginButton_Click(object sender, EventArgs e)
        {
            _loginResult = await _JCoidcClient.LoginAsync(new LoginRequest(), CancellationToken.None);

            if (_loginResult.IsError)
            {
                MessageBox.Show($"Error: {_loginResult.Error}");
            }
            else
            {
                MessageBox.Show($"Success: {_loginResult.User.Identity.AuthenticationType}");
                // Hide the LoginForm form
                Hide();

                // Retrieve the email claim
                var userEmailClaim = _loginResult.User.FindFirst(c => c.Type == "email")?.Value;

                // Pass the user email to the SecureForm constructor
                var secureForm = new secureform(userEmailClaim);
                secureForm.Show();
            }
        }

        private async void OLLoginButton_Click(object sender, EventArgs e)
        {
            _loginResult = await _OLoidcClient.LoginAsync(new LoginRequest(), CancellationToken.None);

            if (_loginResult.IsError)
            {
                MessageBox.Show($"Error: {_loginResult.Error}");
            }
            else
            {
                MessageBox.Show($"Success: {_loginResult.User.Identity.AuthenticationType}");
                // Hide the LoginForm form
                Hide();

                // Retrieve the email claim
                var userEmailClaim = _loginResult.User.FindFirst(c => c.Type == "email")?.Value;

                // Pass the user email to the SecureForm constructor
                var secureForm = new secureform(userEmailClaim);
                secureForm.Show();
            }
        }

        private class SystemBrowser : IBrowser
        {
            private readonly WebBrowser webBrowser;
            private TaskCompletionSource<string> tcs;

            public SystemBrowser(WebBrowser webBrowser)
            {
                this.webBrowser = webBrowser;
            }

            public async Task<BrowserResult> InvokeAsync(BrowserOptions options, CancellationToken cancellationToken = default)
            {
                tcs = new TaskCompletionSource<string>();

                // Handle document completed event
                webBrowser.DocumentCompleted += DocumentCompletedHandler;

                // Navigate to the start URL
                webBrowser.Url = new Uri(options.StartUrl);

                // Wait for the navigation to complete
                var response = await tcs.Task;

                // Return the result
                var result = new BrowserResult
                {
                    Response = response,
                    ResultType = BrowserResultType.Success
                };

                return result;
            }

            private void DocumentCompletedHandler(object sender, EventArgs e)
            {
                var browser = (WebBrowser)sender;

                // Check if the navigation is complete
                if (browser.Url.AbsoluteUri.StartsWith("http://localhost"))
                {
                    // Set the result and remove the event handler
                    tcs.SetResult(browser.Url.AbsoluteUri);
                    browser.DocumentCompleted -= DocumentCompletedHandler;
                }
            }
        }
    }
}